# coding=utf-8

"""
This package gives the functions to interact with Docaster.
"""

# https://aster.retd.edf.fr/scm/hg/others/hgdocaster/log/default/fr/man_u/u4/u4.51.11.odt

import re
from urllib.request import urlopen
from urllib.error import HTTPError
import ssl

from html.parser import HTMLParser
from aslint.logger import logger
from aslint.config import ASCFG
from aslint.string_utils import convert


RE_KEY = re.compile(
    "(?P<key>(?P<chap>(?P<man>[arduvm]){1}[0-9]+)" r"\.[0-9a-zA-Z]+\.[0-9]+)$", re.I
)
RE_TXT = re.compile("<.*?>", re.M)


class MyHTMLParser(HTMLParser):
    """HTML parser to extract changesets descriptions."""

    def __init__(self):
        HTMLParser.__init__(self)
        self._descr = []
        self._buff = None

    def handle_starttag(self, tag, attrs):
        """Search for the beginning of a description."""
        if tag == "td" and ("class", "description") in attrs:
            self._buff = []

    def handle_endtag(self, tag):
        """Search for the end of a description."""
        if tag == "td" and self._buff is not None:
            self._descr.append(" ".join(self._buff).strip())
            self._buff = None

    def handle_data(self, data):
        """Store the description."""
        if self._buff is not None:
            self._buff.append(data.strip())

    def get_descr(self):
        """Return all the descriptions.

        Returns:
            list[str]: List of the descriptions.
        """
        return self._descr


def changelog_doc(key, limit=120):
    """Return the changelogs for a document.

    Arguments:
        key (str): *keyword* of the document (in Docaster terminology, for
            example "u4.32.04").
        limit (int): Maximum number of changesets read.

    Returns:
        list[str]: Descriptions of the changesets.
    """
    keyw = RE_KEY.search(key.lower())
    args = {"limit": limit}
    args.update(keyw.groupdict())

    url = ASCFG.get("docaster_uri").format(**args)
    logger.debug("url: {0}".format(url))
    # do not check certificate - only used on private network
    sctxt = ssl.create_default_context()
    # sctxt.load_verify_locations(ASCFG.get("docaster_cafile"))
    sctxt.check_hostname = False
    sctxt.verify_mode = ssl.CERT_NONE
    try:
        req = urlopen(url, context=sctxt)
    except HTTPError as exc:
        raise ValueError(str(exc))
    content = convert(req.read())
    parser = MyHTMLParser()
    parser.feed(content)
    descr = parser.get_descr()

    return descr
